﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;

namespace HunterCommunication {
    public class HunterComm {
        public int gameID { get; private set; }
        public int maxMoves { get; private set; }
        public int actMoves { get; private set; }
        public int[,] canvas { get; private set; }
        public int p1X { get; private set; }
        public int p1Y { get; private set; }
        public int p1H { get; private set; }
        public int p1P { get; private set; }
        public int p2X { get; private set; }
        public int p2Y { get; private set; }
        public int p2H { get; private set; }
        public int p2P { get; private set; }
        public bool isMultiplayer { get; private set; }

        private string uri = "";
        private int rotation = 0;

        public HunterComm(string url) {
            uri = url;
            refresh(url + "/start");
        }

        public string sendKey(string key) {
            bool doIt = false;
            
            switch (key) {
                case "L":
                    if (p1X - 1 >= 0)
                        if (canvas[p1X - 1, p1Y] != -1 && canvas[p1X - 1, p1Y] != -11) {
                            doIt = true;
                        }
                    break;
                case "P":
                    if (p1X + 1 <= 9)
                        if (canvas[p1X + 1, p1Y] != -1 && canvas[p1X + 1, p1Y] != -11) {
                            doIt = true;
                        }
                    break;
                case "N":
                    if (p1Y - 1 >= 0)
                        if (canvas[p1X, p1Y - 1] != -1 && canvas[p1X, p1Y - 1] != -11) {
                            doIt = true;
                        }
                    break;
                case "D":
                    if (p1Y + 1 <= 9)
                        if (canvas[p1X, p1Y - 1] != -1 && canvas[p1X, p1Y + 1] != -11) {
                            doIt = true;
                        }
                    break;
            }
            if (doIt)
                return refresh(uri + "/tah?id=" + gameID + "&pohyb=" + key);
            return null;
        }

        private string refresh(string url) {
            var webRequest = WebRequest.Create(url);

            using (var response = webRequest.GetResponse())
            using (var content = response.GetResponseStream())
            using (var reader = new StreamReader(content)) {

                string firstLine = reader.ReadLine();
                int memory = 0;
                Int32.TryParse(firstLine, out memory);
                gameID = memory;
                if (gameID != 0) {

                    Int32.TryParse(reader.ReadLine(), out memory);
                    maxMoves = memory;
                    actMoves = memory;

                    canvas = new int[10, 10];
                    StringBuilder sb = new StringBuilder();
                    for (int y = 0; y < 10; y++) {
                        string s = reader.ReadLine();
                        for (int x = 0; x < 10; x++) {
                            sb.Clear();
                            sb.Append(s[x * 2]);
                            sb.Append(s[x * 2 + 1]);
                            switch (sb.ToString()) {
                                case "..":
                                    canvas[x, y] = 0;
                                    break;
                                case "XX":
                                    canvas[x, y] = -1;
                                    break;
                                case "++":
                                    canvas[x, y] = -10;
                                    break;
                                case "--":
                                    canvas[x, y] = -11;
                                    break;
                                default:
                                    Int32.TryParse(sb.ToString(), out canvas[x, y]);
                                    break;
                            }
                        }
                    }

                    string[] readed = reader.ReadLine().Split(' ');

                    Int32.TryParse(readed[0], out memory);
                    p1X = memory - 1;
                    Int32.TryParse(readed[1], out memory);
                    p1Y = memory - 1;
                    Int32.TryParse(readed[2], out memory);
                    p1H = memory;
                    Int32.TryParse(readed[3], out memory);
                    p1P = memory;

                    readed = reader.ReadLine().Split(' ');

                    if (readed.Length == 4) {

                        Int32.TryParse(readed[0], out memory);
                        p2X = memory - 1;
                        Int32.TryParse(readed[1], out memory);
                        p2Y = memory - 1;
                        Int32.TryParse(readed[2], out memory);
                        p2H = memory;
                        Int32.TryParse(readed[3], out memory);
                        p2P = memory;

                        isMultiplayer = true;
                    }
                } else {
                    if (firstLine.StartsWith("Konec hry: ")) {
                        return firstLine;
                    }
                }
                return null;
            }
        }

        public string playAuto() {
            List<List<Tuple<int, int>>> paths = new List<List<Tuple<int,int>>>();
            List<List<Tuple<int, int>>> pathsNew = new List<List<Tuple<int, int>>>();
            List<Tuple<int, int>> usedPlaces = new List<Tuple<int, int>>();
            usedPlaces.Add(new Tuple<int, int>(p1X, p1Y));
            paths.Add(new List<Tuple<int, int>> { new Tuple<int, int>(p1X, p1Y) });

            while (!(usedPlaces.Where(e => canvas[e.Item1, e.Item2] > 0).Count() > 0)) {
                pathsNew = new List<List<Tuple<int, int>>>();
                foreach (List<Tuple<int, int>> l in paths) {
                    if (check(l.Last().Item1 + 1, l.Last().Item2) && !(usedPlaces.Contains(new Tuple<int, int>(l.Last().Item1 + 1, l.Last().Item2)))) {
                        List<Tuple<int, int>> adding = new List<Tuple<int, int>>();
                        adding.AddRange(l);
                        pathsNew.Add(adding);
                        pathsNew.Last().Add(new Tuple<int, int>(l.Last().Item1 + 1, l.Last().Item2));
                        usedPlaces.Add(new Tuple<int, int>(l.Last().Item1 + 1, l.Last().Item2));
                    }
                    if (check(l.Last().Item1 - 1, l.Last().Item2) && !(usedPlaces.Contains(new Tuple<int, int>(l.Last().Item1 - 1, l.Last().Item2)))) {
                        List<Tuple<int, int>> adding = new List<Tuple<int, int>>();
                        adding.AddRange(l);
                        pathsNew.Add(adding);
                        pathsNew.Last().Add(new Tuple<int, int>(l.Last().Item1 - 1, l.Last().Item2));
                        usedPlaces.Add(new Tuple<int, int>(l.Last().Item1 - 1, l.Last().Item2));
                    }
                    if (check(l.Last().Item1, l.Last().Item2 + 1) && !(usedPlaces.Contains(new Tuple<int, int>(l.Last().Item1, l.Last().Item2 + 1)))) {
                        List<Tuple<int, int>> adding = new List<Tuple<int, int>>();
                        adding.AddRange(l);
                        pathsNew.Add(adding);
                        pathsNew.Last().Add(new Tuple<int, int>(l.Last().Item1, l.Last().Item2 + 1));
                        usedPlaces.Add(new Tuple<int, int>(l.Last().Item1, l.Last().Item2 + 1));
                    }
                    if (check(l.Last().Item1, l.Last().Item2 - 1) && !(usedPlaces.Contains(new Tuple<int, int>(l.Last().Item1, l.Last().Item2 - 1)))) {
                        List<Tuple<int, int>> adding = new List<Tuple<int, int>>();
                        adding.AddRange(l);
                        pathsNew.Add(adding);
                        pathsNew.Last().Add(new Tuple<int, int>(l.Last().Item1, l.Last().Item2 - 1));
                        usedPlaces.Add(new Tuple<int, int>(l.Last().Item1, l.Last().Item2 - 1));
                    }
                }
                paths = pathsNew;
            }

            return null;
        }

        private bool check(int x, int y) {
            if ((!(x < 0 || x > 9)) && (!(y < 0 || y > 9)))
                if (!(canvas[x, y] == -1 || canvas[x, y] == -11))
                    return true;
            return false;
        }
    }
}
